import urllib2
import json

import logging
import optparse
import subprocess

import gettext
from gettext import gettext as _
gettext.textdomain('unity-lens-rubygems')

from singlet.lens import SingleScopeLens, IconViewCategory, ListViewCategory

from unity_lens_rubygems import unity_lens_rubygemsconfig

class RubygemsLens(SingleScopeLens):

    class Meta:
        name = 'rubygems'
        description = _('Rubygems Lens')
        search_hint = _('Search Rubygems')
        icon = 'unity-lens-rubygems.svg'
        search_on_blank = False

    available_rgems_category = ListViewCategory(_("Available Rubygems"), 'text-x-ruby')
    installed_rgems_category = IconViewCategory(_("Installed Rubygems"), 'text-x-ruby')

    def shorten_desc(self, desc):
        if len(desc) > unity_lens_rubygemsconfig.DESC_MAX_LENGTH:
            return "%s..." % desc[:unity_lens_rubygemsconfig.DESC_MAX_LENGTH]
        else:
            if desc[-1:] != ".":
                return desc + "."
            else:
                return desc

    def gem_search_query(self, search):
        gems_api_url = "https://rubygems.org/api/v1"
        try:
            data = json.load(urllib2.urlopen("%s/search.json?query=%s"%(gems_api_url,search)))
            return data
        except Exception:
            return []

    def gem_installed_query(self, search):
        try:
            new_data = []
            data = subprocess.check_output(["gem", "search", search], universal_newlines=True).splitlines()

            for line in data:
                if len(line) > 0 and "local gems" not in line.lower():
                    new_data.append(line.split(" ")[0])
            return new_data
        except Exception:
            return []

    def search(self, search, results):
        for gem in self.gem_search_query(search):
            results.append(gem["homepage_uri"],
                "applications-ruby",
                self.available_rgems_category,
                "text/html",
                gem["name"],
                self.shorten_desc(gem["info"]),
                gem["homepage_uri"])

        for gem in self.gem_installed_query(search):
            results.append("http://rubygems.org/gems/%s" % gem,
                "applications-ruby",
                self.installed_rgems_category,
                "text/html",
                gem,
                _("Locally installed rubygem"),
                "http://rubygems.org/gems/%s" % gem)
        pass
